#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <ctype.h>
#include "hash.h"


/* Add entry to the list */
#define add_entry(_head, _entry, _typename) \
    _add_entry(_head, _entry, offsetof(_typename, bucket), sizeof(_typename))

static void _add_entry(
    void   *head,
    void   *entry,
    off_t   next_offset,
    size_t  entry_size)
{
    void **itr;

    if (!head || !entry)
      return;

    /* First entry */
    if (head == entry)
      return;

    itr = head + next_offset;
    while ((char *)(*itr))
      itr = *itr + next_offset;

    /* Add to the end of the list */
    *itr = entry;
}


hash_key_t hash_default_hash(const char *data, size_t data_sz)
{
    int           i;
    unsigned long val;

    for (i=0, val=0; i<data_sz; ++i)
      val += data[i];

    return (hash_key_t)(val % HASH_DEFAULT_SIZE);
}


hash_t *hash_create(int n_keys)
{
    return calloc(1, sizeof(hash_t) * n_keys);
}


void hash_add(hash_t *hash, const void *data, size_t data_sz, hash_key_t key)
{
    hash_t *buck, *hh;

    hh = &hash[key];

    if (hh->data)
    {
        buck = hash_create(1);
        buck->data = data;
        buck->data_sz = data_sz;

        if (hh->bucket)
          add_entry(hh, buck, hash_t);
        else
          hh->bucket = buck;
    }
    else
    {
        hh->data = data;
        hh->data_sz = data_sz;
    }
}


void hash_print_entries_as_strings(const hash_t *hash, int n_keys)
{
    int i;

    for (i=0; i<n_keys; ++i)
    {
        if (hash[i].data)
          printf("[%i] %s\n", i, (char *)hash[i].data);

        if (hash[i].bucket)
          hash_print_entries_as_strings(hash[i].bucket, 1);
    }
}


int hash_matches(const hash_t *hash, void *data, size_t data_sz)
{
    const hash_t *hh = &(hash[hash_default_hash(data, data_sz)]);

    while (hh)
      if (hh->data && (memcmp(data, hh->data, data_sz) == 0))
        return 1;
      else
        hh = hh->bucket;

    return 0;
}
