#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "main.h"
#include "hash.h"


static void usage(char **argv)
{
    printf("[[ Slayerize v%s ]]\n", VERSION);
    printf("Usage: %s <-u rss URL> <-l lyrics file> [-v verbosity]\n",argv[0]);
    exit(0);
}


static double rating(rss_t *rss, int n_rss, const hash_t *lyrics)
{
    int         i, j, is_match, valid_word, n_words, n_matches, word_idx;
    double      rating;
    char        word[RSS_TITLE_LENGTH];
    const char *c;

    rating = 0.0;
    n_words = n_matches = 0;
    for (i=0; i<n_rss; ++i)
    {
        word_idx = 0;
        c = rss[i].title;
        for (j=0; j<RSS_TITLE_LENGTH; ++j, ++c)
        {
            if (isspace(*c))
            {
                word[word_idx] = '\0';
                valid_word = ((word_idx > 0) && (word[0] != '-')) ? 1 : 0;
                is_match =
                  (valid_word && hash_matches(lyrics, word, word_idx)) ? 1 : 0;

                /* Stats */
                n_words += valid_word;
                rss[i].n_words += valid_word;
                n_matches += is_match;
                rss[i].n_matches += is_match;

                /* Save matched word for later */
                if (is_match)
                {
                    strcat(rss[i].matched_words, word);
                    strcat(rss[i].matched_words, ":");
                }
                
                word_idx = 0;
            }
            else if (islower(*c) || isupper(*c) || (*c == '-') || (*c == '\''))
              word[word_idx++] = tolower(*c);
        }
    }

    return ((double)n_matches / n_words);
}


static void output_matches(const rss_t *rss, int n_rss)
{
    int i;

    for (i=0; i<n_rss; ++i)
      printf("<%f>,<%s>,<%s>,<%s>\n",
             (double)rss[i].n_matches/rss[i].n_words,
             rss[i].title, rss[i].url, rss[i].matched_words);
}


int main(int argc, char **argv)
{
    int     i, n_rss, verbose;
    char   *url, *lyrics_fname;
    double  slayer;
    rss_t  *rss;
    hash_t *lyrics;

    verbose = 0;
    url = lyrics_fname = NULL;

    for (i=1; i<argc; ++i)
    {
        if (argv[i][0] != '-')
          usage(argv);

        if (argv[i][1] == 'v')
          verbose = 1;
        else if ((argv[i][1] == 'u') && (i+1<argc))
           url = argv[++i];
         else if ((argv[i][1] == 'l') && (i+1<argc))
           lyrics_fname = argv[++i];
         else
           usage(argv);
    }

    if (!url || !lyrics)
      usage(argv);
    
    rss = utils_get_rss(url, &n_rss);
    lyrics = utils_get_lyrics(lyrics_fname, NULL);
    slayer = rating(rss, n_rss, lyrics);

    if (verbose)
      output_matches(rss, n_rss);

    printf("====[ %f ]====\n", slayer);

    return 0;
}
